/*    
******************************************************************************
	PVGA1 IO functions and variables
******************************************************************************

   Copyright (C) 1997 by Brian S. Julin, <bri@calyx.com>

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************
*/

#ifndef _pvga1a_io_c
#define _pvga1a_io_c

/* MAINTAINER	Brian S. Julin, <bri@calyx.com> */

#include <WD/pvga1a_io.h>

/**** Reserved PIO ports and MMIO regions ****/

struct io_region wd_miscw = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  MISCw,		/* IO region start (physical) 
			   (gets filled in below)	*/
  0x01,			/* IO region size		*/
  0x3FF,		/* 10 address bits are decoded	*/
  "KGI wd MISC0/ISR0"   /* IO region name		*/
};

struct io_region wd_miscr = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  MISCr,		/* IO region start (physical) 
			   (gets filled in below)	*/
  0x01,			/* IO region size		*/
  0x3FF,		/* 10 address bits are decoded	*/
  "KGI wd MISC read"    /* IO region name		*/
};

struct io_region wd_crtc = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  CRTIc,	        /* IO region start (physical) 
	       		   (changed below if mono)	*/
  0x02,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd CRTC"	        /* IO region name		*/
};

struct io_region wd_seq = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  SEQI,		        /* IO region start (physical) 
			   (gets filled in below)	*/
  0x02,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Sequencer"     /* IO region name		*/
};

struct io_region wd_grc = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  GRCI,		        /* IO region start (physical) 
			   (gets filled in below)	*/
  0x02,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Graph. Cont."  /* IO region name		*/
};

struct io_region wd_atrc = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  ATCI,		        /* IO region start (physical) 
			   (gets filled in below	*/
  0x01,			/* IO region size (changed later for 16bit) */
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Attrib. Cont." /* IO region name		*/
};

struct io_region wd_wakeup = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  WD_WAKEUP,		/* IO region start (physical)	*/
  0x01,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Wakeup"	/* IO region name		*/
};

struct io_region wd_sleep_nonisa = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  WD_SLEEP_NONISA,	/* IO region start (physical)	*/
  0x01,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Sleep"       	/* IO region name		*/
};

struct io_region wd_sleep = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  WD_SLEEP,		/* IO region start (physical)	*/
  0x01,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Sleep"       	/* IO region name		*/
};

struct io_region wd_sleep2 = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  WD_SLEEP2,		/* IO region start (physical)	*/
  0x20,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Sleep(aux)"	/* IO region name		*/
};

struct io_region wd_sleep3 = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  WD_SLEEP3,		/* IO region start (physical)	*/
  0x20,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Sleep(aux)"	/* IO region name		*/
};

struct io_region wd_sleep4 = 
{
  IO_NULL,		/* IO region start (virtual)	*/
  WD_SLEEP4,		/* IO region start (physical)	*/
  0x20,			/* IO region size		*/
  0x3FF,	       	/* 10 address bits are decoded	*/
  "KGI wd Sleep(aux)"	/* IO region name		*/
};

struct mem_region wd_mem_graph = 
{
  MEM_NULL,		/* Mem region start (virtual)	*/
  0xA0000,		/* Mem region start (physical)	*/
  0x10000,		/* Graphics (64K) region	*/
  -1,			/* Decode all address bits	*/
  "KGI wd - graphics"	/* Mem region name		*/
};

#ifndef TEXT_A000
struct mem_region wd_mem_text = 
{
	MEM_NULL,		/* Mem region start (virtual)	*/
	0xB8000,		/* Mem region start (physical)	*/
	0x08000,		/* Text (32K) region		*/
	-1,			/* Decode all address bits	*/
	"KGI wd - textmode"	/* Mem region name		*/
};
#endif


/**** MMIO MMAP Register Manipulation ****/

/* Set paged linear regsiter A */
#ifndef _wd90c3x_io_c
void outPR0A(kgiu16 page)   /* wd90c33 has a ninth bit */
{
  kgiu8 regval;
  regval = rwunlockGRC();
  outbGRC((kgiu8)(page & 0xFF), GRC_PR0A);
  outbGRC(regval, GRC_UPGRC_LOCK);
}
#endif /* not _wd90c3x_io_c */

/* Set paged linear  register B*/
void outPR0B(kgiu16 page)
{
  kgiu8 regval;
  regval = rwunlockGRC();
  outbGRC((kgiu8)(page & 0xFF), GRC_PR0B);
  outbGRC(regval, GRC_UPGRC_LOCK);
}

void save_GRC(kgiu8 *buf)
{
  int i, lockreg = 0;
  
  for (i = 0; i < NoGRCRegs; i++) 
    {
      if(i == GRC_MEMORY) {
	/* Have to unlock for this register on up */
	/* Can't read rlock, so leave locked */
	lockreg = inbGRC(GRC_UPGRC_LOCK) | GR0F_RLOCK;
	outbGRC((lockreg & ~GR0F_RLOCK), GRC_UPGRC_LOCK);
      }
      *(buf++) = inbGRC(i);
    }
  outbGRC(lockreg, GRC_UPGRC_LOCK);
}

void restore_GRC(kgiu8 *buf)
{
  register kgiu8 i, lockreg = 0; 
  
  for (i = 0; i < NoGRCRegs; i++) {
    if(i == GRC_MEMORY) {
      lockreg = rwunlockGRC();
    }    
    outbGRC(*(buf++), i);
  }
  outbGRC(lockreg, GRC_UPGRC_LOCK);
}


/**** Attribute Controller Register Manipulation ****/

void save_ATC(kgiu8 *buf)
{
  int i;
  
  for (i = 0; i < NoATCRegs; i++) 
    {
      io_in8(INS1r);  /* Reset AFF and setup index */
      *(buf++) = inbATC(i);
    }
  io_in8(INS1r); 
  io_out8(ATCI_ENABLEDISPLAY, ATCI);
}

void restore_ATC(kgiu8 *buf)
{
  register kgiu8 i;
  
  for (i = 0; i < NoATCRegs; i++) {
    io_in8(INS1r); 
    outbATC(*(buf++), i);
  }
  io_in8(INS1r); 
  io_out8(ATCI_ENABLEDISPLAY, ATCI);
}


/**** CRT Controller Register Manipulation ****/
#ifndef _wd90c00_io_c
void save_CRT(kgiu8 *buf)   
{
  register kgiu8 i;
  for (i = 0; i < NoCRTRegs; i++) 
    {
      *(buf++) = inbCRT(i);
    }
}

void restore_CRT(kgiu8 *buf)  
{
  register kgiu8 i;
  int lockval1, lockval2; 
  lockval1 = wunlockCRT();

  outbCRT(inbCRT(CRT_HBLANKEND) | CR03_IS_VGA,CRT_HBLANKEND);
    
  for (i = 0; i < NoCRTRegs; i++) 
    {
      switch (i) 
	{
	case CRT_HBLANKEND:
	  *buf |= CR03_IS_VGA;       /* FIXME! -- modifies input buffer */
	  break;
	case CRT_VSYNCEND:
	  *buf &= ~CR11_LOCKTIMING;  /* FIXME! -- modifies input buffer */
	  break;
	default:
	  break;
	}
      outbCRT(*(buf++), i);
    }

  lockval2 = wunlockGRC();           /* Unlock locking register */
  /* It gets worse -- see code for later chipsets :-) */
  outbGRC(lockval1, GRC_CRTC_CTL);   /* Restore lock state */
  outbGRC(lockval2, GRC_UPGRC_LOCK); /* Restore upper GRC locking state */
}
#endif /* not _wd90c00_io_c */

/**** Sequencer Register Manipulation ****/

void save_SEQ(kgiu8 *buf) 
{
  int i;
  for (i = 0; i < NoSEQRegs; i++) 
    {      
      *(buf++) = inbSEQ(i);
    }
}

void restore_SEQ(kgiu8 *buf) 
{
  register kgiu8 i;
  for (i = 0; i < NoSEQRegs; i++) 
    {
      outbSEQ(*(buf++), i);
    }
}

void save_state(struct statetable *state) 
{
  state->MISC = inbMISC();
  save_SEQ(state->SEQ);	
  save_CRT(state->CRT);	
  save_GRC(state->GRC);	
  save_ATC(state->ATC);	
}

void restore_state(struct statetable *state)
{
  int lockval1,lockval2;
  lockval1 = wunlockMISC();

  outbMISC(state->MISC);

  lockval2 = wunlockGRC();           /* Unlock locking register */
  outbGRC(lockval1, GRC_CRTC_CTL);   /* Restore lock state */
  outbGRC(lockval2, GRC_UPGRC_LOCK); /* Restore upper GRC locking state */

  restore_SEQ(state->SEQ);
  restore_CRT(state->CRT);
  restore_GRC(state->GRC);
  restore_ATC(state->ATC);

}

/* Functions used in chipset detection -- these do not manipulate locks */

int iswrite_idx_reg(kgiu8 bitmask, kgiu16 idx, kgiu16 dat, kgiu8 reg) 
{
  int old;
  old = inb_idx_reg(idx,dat,reg);
  outb_idx_reg(old & (~ bitmask), idx, reg);
  if ((inb_idx_reg(idx, dat, reg) & bitmask) == 0) 
    {
      outb_idx_reg(old | bitmask, idx, reg);
      if ((inb_idx_reg(idx, dat, reg) & bitmask) == bitmask) 
	{
	  outb_idx_reg(old,idx,reg);
	  return(TRUE);
	}
    }
  outb_idx_reg(old, idx, reg);
  return (FALSE);
}

int iswriteATC(kgiu8 bitmask, kgiu8 reg) 
{
  int old;
  old = inbATC(reg);
  outbATC(old & (~ bitmask), reg);
  if ((inbATC(reg) & bitmask) == 0) 
    {
      outbATC(old | bitmask, reg);
      if ((inbATC(reg) & bitmask) == bitmask) 
	{
	  outbATC(old,reg);
	  return(TRUE);
	}
    }
  outbATC(old,reg);
  return (FALSE);
}

/********************************************************************************/
/*                          RAM DAC output functions                     */
/********************************************************************************/

/* These are KGI module interface, so cannot be inline */
kgiu16 dac_port[4] = { 0x3C8, 0x3C9, 0x3C6, 0x3C7 };

inline void DAC_out8(struct kgi_display *dpy, 
		     kgiu8 value, kgiu8 dac_reg)
{ io_out8(value, dac_port[dac_reg & 0x03]); }

kgiu8 DAC_in8(struct kgi_display *dpy, kgiu8 dac_reg)
{ return io_in8(dac_port[dac_reg & 0x03]); }

void DAC_ins8(struct kgi_display *dpy, kgiu8 dac_reg,
	      void *buf, kgi_uint count)
{ io_ins8(dac_port[dac_reg & 0x03], buf, count); }

void DAC_outs8(struct kgi_display *dpy, kgiu8 dac_reg, 
	       void *buf, kgi_uint count)
{ io_outs8(dac_port[dac_reg & 0x03], buf, count); }


/* Clock IO functions, also part of the KGI module interface */

void CLK_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 reg)
{	
  kgiu8 regval;
  /*
  if ((reg != 0) | (value > 0xf))
    {	
      INTERNAL_ERROR;
    }
  */
  regval = inbMISC();
  regval = ((regval & ~MISC_CLOCK_SEL_B0) | 
	    ((value & 0x01) ? MISC_CLOCK_SEL_B0 : 0));
  regval = ((regval & ~MISC_CLOCK_SEL_B1) | 
	    ((value & 0x02) ? MISC_CLOCK_SEL_B1 : 0));
  outbMISC(regval); 

/* Put this back in when getCLK issue resolved 
	 and wd90c1x driver added 
  regval = rwunlockGRC();

	outbGRC(((inbGRC(GRC_VID_CONF) & ~GR0C_CLOCK_SEL_B2) |
					 ((value & 0x04) ? GR0C_CLOCK_SEL_B2 : 0)),
					GRC_VID_CONF); 

  outbGRC(regval, GRC_UPGRC_LOCK);
	*/
}

kgiu8 CLK_in8(struct kgi_display *dpy, kgiu8 reg)
{
  kgiu8 result, lockreg;
  /*  if (reg != 0)
    { 
      INTERNAL_ERROR;
    } 
    */
  result = 0;

  result += (inbMISC() & MISC_CLOCK_SEL_B0) ? 1 : 0;
  result += (inbMISC() & MISC_CLOCK_SEL_B1) ? 2 : 0;

/* Put this back in when getCLK issue resolved 
	 and wd90c1x driver added 
  lockreg = runlockGRC();
	
	result += (inbGRC(GRC_VID_CONF) & GR0C_CLOCK_SEL_B2) ? 4 : 0;
	
  outbGRC(lockreg, GRC_UPGRC_LOCK);
	
  return(result);
	*/

}


#ifdef FOOBARRR
void dump_state(struct statetable *state)
{ 
  int i;
	printk("MISC: %.2x", state->MISC);
	
	for (i = 0; i < NoSEQRegs; i++)
	  { 
	    if(!(i & 0xF))
	      printk("\nSEQ:%.2x: ", i);
		printk("%.2x, ", state->SEQ[i]);
	  }
	
	for (i = 0; i < NoCRTRegs; i++)
	{ 
	  if(!(i & 0xf))
	    printk("\nCRT:%.2x: ", i);
		printk("%.2x, ", state->CRT[i]);
	}
	
	for (i = 0; i < NoGRCRegs; i++)
	  { 
	    if(!(i & 0xF)) 
	      printk("\nGRC:%.2x: ", i);
	    printk("%.2x, ", state->GRC[i]);
	  }
	
	for (i = 0; i < NoATCRegs; i++)
	  { 
	    if(!(i & 0xF)) 
	      printk("\nATC:%.2x: ", i);
	    printk("%.2x, ", state->ATC[i]);
	}
	printk("\n");
}


#endif 

#endif /* notdef _pvga1a_io_c */
