/* $Id: accel.h,v 1.3 1998/11/24 18:45:21 marcus Exp $
******************************************************************************

   Tseng Labs ET4000/ET6000 acceleration definitions

   Copyright (c) 1997		Jim Ursetto	[jim.ursetto@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/


#ifndef _TSENG_ACCEL_H
#define _TSENG_ACCEL_H

#define MMU_BASEPTR0 0x00     /* MMU Apertures 0 - 2 (DWORD aligned for ET6000) */
#define MMU_BASEPTR1 0x04
#define MMU_BASEPTR2 0x08
#define MMU_CONTROL  0x13
#define ACL_SUSPEND_TERMINATE 0x30   /* Suspend/Terminate Operation */
  #define ACL30_SUSPEND  0x01
  #define ACL30_TERMINATE 0x10
#define ACL_OPERATION_STATE 0x31   
  #define ACL31_START_ENABLE 0x10
#define ACL_SYNC_ENABLE  0x32
#define ACL_CONFIG_6000  0x32    /* Configuration - ET6000 only */
#define ACL_INTERRUPT_MASK 0x34  /* Enable/disable read/write interrupts */
#define ACL_INTERRUPT_STATUS 0x35
#define ACL_ACCELERATOR_STATUS 0x36
  #define ACL36_BUSY 0x02
  #define ACL36_XYUNSTABLE 0x04
#define ACL_POWER_CONTROL 0x37   /* Power down engine when idle - ET6000 only */
  #define ACL37_DOWN_WHEN_IDLE 0x01
/* 6000 and W32p use these */
#define ACL_X_POSITION 0x38      /* Accelerator's X position */
#define ACL_Y_POSITION 0x3a      /* Accelerator's Y position */
/* W32 and W32i use these */
#define ACL_X_POS_W32 0x94      /* Accelerator's X position */
#define ACL_Y_POS_W32 0x96      /* Accelerator's Y position */

#define ACL_PATTERN_ADDRESS 0x80  /* Address in display memory of pattern map */
#define ACL_SOURCE_ADDRESS  0x84  /* Address in display memory of source map */
#define ACL_PATTERN_Y_OFFSET 0x88 /* # of bytes - 1 between lines in pattern */
#define ACL_SOURCE_Y_OFFSET 0x8A  /* # of bytes - 1 between lines in source */
#define ACL_DEST_Y_OFFSET 0x8C  /* # of bytes - 1 between lines in dest */

#define ACL_VIRTUAL_BUS_SIZE 0x8E /* W32, W32i only */
#define ACL_PIXEL_DEPTH 0x8E      /* W32p, 6000 */
#define ACL_XY_DIRECTION 0x8F  /* X & Y dir, graphics opcode, line drawing */

#define ACL_PATTERN_WRAP 0x90  /* Pattern X and Y wrap */
#define ACL_XFER_DISABLE 0x91  /* Disable register transfers - 6000 only */
#define ACL_SOURCE_WRAP 0x92   /* Source X and Y wrap */

#define ACL_SECONDARY_EDGE 0x93  /* 6000 only */
#define ACL_X_COUNT 0x98 /* Number of bytes - 1 in blit area, times bpp */
#define ACL_Y_COUNT 0x9A /* Number of lines - 1 in the blit area */

#define ACL_ROUTING_CONTROL 0x9C /* not 6000 */
#define ACL_MIX_CONTROL 0x9C /* Mix/Mask raster op select - 6000 only */
#define ACL_RELOAD_CONTROL 0x9D /* not 6000 */
#define ACL_STEP_INHIBIT 0x9D /* Inhibit X or Y stepping of maps */
#define ACL_BG_RASTER_OP 0x9E
#define ACL_FG_RASTER_OP 0x9F

#define ACL_DEST_ADDRESS 0xA0 /* Address of Dest. Map in display memory */
#define ACL_MIX_ADDRESS  0xA4 /* Address of the MixMap */
#define ACL_MIX_Y_OFFSET 0xA8 /* # of bits - 1 per scan line in the MixMap */

/* Bresenham linedrawing registers */
#define ACL_ERROR_TERM 0xAA  
#define ACL_DELTA_MINOR 0xAC /* Bresenham */
#define ACL_DELTA_MAJOR 0xAE /* Bresenham */
/* Trapezoid registers - 6000 only */
#define ACL_SECONDARY_ERROR_TERM 0xB2
#define ACL_SECONDARY_DELTA_MINOR 0xB4
#define ACL_SECONDARY_DELTA_MAJOR 0xB6

#endif /* !defined _TSENG_ACCEL_H */
