/* $Id: init.c,v 1.1 2000/12/10 11:31:36 marcus Exp $
******************************************************************************

   GGLGL implementation for X target

   Copyright (C) 2000 Marcus Sundberg	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include "xggigl.h"


static void SwapBuffers(ggi_visual *vis)
{
	glXSwapBuffers(GGIX_PRIV(vis)->xwin.x.display,
		       GGIX_PRIV(vis)->xwin.window);
}

static int GGIGLopen(ggi_visual *vis, struct ggi_dlhandle *dlh,
		     const char *args, void *argptr, uint32 *dlret)
{
	ggiglpriv *extpriv = GGIGLEXT_PRIV(vis);
	Display *disp = GGIX_PRIV(vis)->xwin.x.display;
	Window win = GGIX_PRIV(vis)->xwin.window;
	XVisualInfo *vinfo;
	int attrib[32];
	int i = 0;

	XGGIGL_PRIV(vis) = malloc(sizeof(struct xggigl_priv));
	if (XGGIGL_PRIV(vis) == NULL) return GGI_ENOMEM;

	/* Setup GLX attributes */
        attrib[i++] = GLX_RGBA;
#if 0
	attrib[i++] = GLX_RED_SIZE;
	attrib[i++] = 1;
	attrib[i++] = GLX_GREEN_SIZE;
	attrib[i++] = 1;
	attrib[i++] = GLX_BLUE_SIZE;
	attrib[i++] = 1;
	attrib[i++] = GLX_DOUBLEBUFFER;
#endif
	attrib[i++] = GLX_DEPTH_SIZE;
	attrib[i++] = 16;

	attrib[i] = None;

	vinfo = glXChooseVisual(disp, win, attrib);
	if (vinfo == NULL) {
		free(XGGIGL_PRIV(vis));
		return GGI_ENODEVICE;
	}

	XGGIGL_PRIV(vis)->context = glXCreateContext(disp, vinfo, NULL, True);

	glXMakeCurrent(disp, win, XGGIGL_PRIV(vis)->context);

	extpriv->swapbuffers = SwapBuffers;

	*dlret = GGI_DL_EXTENSION;
	return 0;
}


static int GGIGLexit(ggi_visual *vis, struct ggi_dlhandle *dlh)
{
	glXDestroyContext(GGIX_PRIV(vis)->xwin.x.display,
			  XGGIGL_PRIV(vis)->context);

	free(XGGIGL_PRIV(vis));

	XGGIGL_PRIV(vis) = NULL;

	return 0;
}


int GGIGLdl_X(int func, void **funcptr)
{
	switch (func) {
	case GGIFUNC_open:
		*funcptr = GGIGLopen;
		return 0;
	case GGIFUNC_exit:
		*funcptr = GGIGLexit;
		return 0;
	case GGIFUNC_close:
		*funcptr = NULL;
		return 0;
	default:
		*funcptr = NULL;
	}

	return GGI_ENOTFOUND;
}

#include <ggi/internal/ggidlinit.h>
